"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrimaryHDU = void 0;
const fs_1 = require("fs");
const fits_structure_1 = require("./fits-structure");
class Filter {
    static basicFilter(structure) {
        const stats = structure.getDataStats();
        const max = stats.median + stats.stdDev;
        const min = stats.median - stats.stdDev;
        return {
            filter: (value) => {
                return Math.min(Math.max(value, min), max);
            },
        };
    }
    filter(value) {
        throw new Error("Not implemented");
    }
}
class PrimaryHDU {
    constructor(fileName) {
        this.stats = [];
        this.fileName = fileName;
    }
    static getFileBuffer(fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            const buffer = yield new Promise((resolve, reject) => (0, fs_1.readFile)(fileName, (error, data) => {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(data);
                }
            }));
            if (!buffer) {
                throw new Error(`Unable to load ${fileName}`);
            }
            return buffer;
        });
    }
    static fromFile(fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield new PrimaryHDU(fileName).load();
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.primaryHDU) {
                this.primaryHDU = new fits_structure_1.FitsStructure(yield PrimaryHDU.getFileBuffer(this.fileName));
                this.structures = [];
                let next = this.primaryHDU.getNextStructure();
                while (next) {
                    this.structures.push(next);
                    next = next.getNextStructure();
                }
            }
            return this;
        });
    }
    getHdu() {
        if (!this.primaryHDU) {
            throw new Error("Must await `load` first");
        }
        return this.primaryHDU;
    }
    getStructure(layerNum) {
        return this.getStructures()[layerNum];
    }
    getStructures() {
        if (!this.structures) {
            throw new Error("Must await `load` first");
        }
        return this.structures;
    }
    getImageStructures() {
        if (this.images) {
            return this.images;
        }
        this.images = this.getStructures().filter((structure) => structure.isImage());
        return this.images;
    }
    getLayerStats(layerNum) {
        if (this.stats[layerNum]) {
            return this.stats[layerNum];
        }
        this.stats[layerNum] = this.getStructure(layerNum).getDataStats();
        return this.stats[layerNum];
    }
    filterLayer(layerNum, filter) {
        filter = filter || Filter.basicFilter(this.getStructure(layerNum));
        const filtered = [];
        const data = this.getStructure(layerNum).getDataValues();
        data.forEach((value) => {
            if (!filter) {
                throw new Error("No filter");
            }
            filtered.push(filter.filter(value));
        });
        return filtered;
    }
}
exports.PrimaryHDU = PrimaryHDU;
//# sourceMappingURL=primary-hdu.js.map